@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -33,6 +32,7 @@
 
 """Python function to register |UDM| extensions in |LDAP|."""
 
+from __future__ import annotations
 
 import base64
 import bz2
@@ -48,19 +48,14 @@ import tempfile
 import time
 from abc import ABCMeta, abstractmethod, abstractproperty
 from copy import copy
-from optparse import Option, OptionGroup, OptionParser, OptionValueError, Values  # noqa: F401
-from typing import TYPE_CHECKING, Dict, List, Optional, Tuple  # noqa: F401
+from optparse import Option, OptionGroup, OptionParser, OptionValueError, Values
+from typing import TYPE_CHECKING
 
 import apt
 from ldap.dn import escape_dn_chars
 from ldap.filter import filter_format
 
 import univention.admin as udm
-
-
-if TYPE_CHECKING:
-    import univention.admin.handlers as udm_handlers  # noqa: F401
-
 import univention.debug as ud
 from univention.admin import modules as udm_modules, uexceptions as udm_errors, uldap as udm_uldap
 from univention.config_registry import ConfigRegistry, configHandlers
@@ -70,12 +65,15 @@ from univention.lib.umc_module import MIME_DESCRIPTION
 import listener
 
 
+if TYPE_CHECKING:
+    import univention.admin.handlers as udm_handlers
+
+
 class BaseDirRestriction(Exception):
     pass
 
 
-def safe_path_join(basedir, filename):
-    # type: (str, str) -> str
+def safe_path_join(basedir: str, filename: str) -> str:
     path = os.path.join(basedir, filename)
     if not os.path.abspath(path).startswith(basedir):
         raise BaseDirRestriction('filename %r invalid, not underneath of %r' % (filename, basedir))
@@ -84,14 +82,13 @@ def safe_path_join(basedir, filename):
     return path
 
 
-def _verify_handler_message_container(lo, position):
-    # type: (udm_uldap.access, udm_uldap.position) -> None
+def _verify_handler_message_container(lo: udm_uldap.access, position: udm_uldap.position) -> None:
     position_dn = 'cn=univention,{}'.format(listener.configRegistry.get('ldap/base'))
     udm_modules.update()
     cn_module = udm_modules.get('container/cn')
     udm_modules.init(lo, position, cn_module)
     try:
-        cn_object = cn_module.object(None, lo, position, dn='cn=handler_messages,{}'.format(position_dn))
+        cn_object = cn_module.object(None, lo, position, dn=f'cn=handler_messages,{position_dn}')
     except udm_errors.noObject:
         position.setDn(position_dn)
         cn_object = cn_module.object(None, lo, position)
@@ -100,13 +97,12 @@ def _verify_handler_message_container(lo, position):
         cn_object.create()
 
 
-def _get_handler_message_object(lo, position, handler_name, create=False):
-    # type: (udm_uldap.access, udm_uldap.position, str, bool) -> udm_handlers.simpleLdap
+def _get_handler_message_object(lo: udm_uldap.access, position: udm_uldap.position, handler_name: str, create: bool = False) -> udm_handlers.simpleLdap:
     position_dn = 'cn=handler_messages,cn=univention,{}'.format(listener.configRegistry.get('ldap/base'))
     udm_modules.update()
     data_module = udm_modules.get('settings/data')
     udm_modules.init(lo, position, data_module)
-    data_object_dn = 'cn={},{}'.format(escape_dn_chars(handler_name), position_dn)
+    data_object_dn = f'cn={escape_dn_chars(handler_name)},{position_dn}'
     try:
         data_object = data_module.object(None, lo, position, dn=data_object_dn)
     except udm_errors.noObject:
@@ -120,11 +116,10 @@ def _get_handler_message_object(lo, position, handler_name, create=False):
     return data_object
 
 
-def set_handler_message(name, dn, msg):
-    # type: (str, str, str) -> None
+def set_handler_message(name: str, dn: str, msg: str) -> None:
     # currently only on Primary Directory Node
     if listener.configRegistry.get('server/role') in ('domaincontroller_master',):
-        ud.debug(ud.LISTENER, ud.INFO, 'set_handler_message for {}'.format(name))
+        ud.debug(ud.LISTENER, ud.INFO, f'set_handler_message for {name}')
         setuid = os.geteuid() != 0
         if setuid:
             listener.setuid(0)
@@ -151,8 +146,7 @@ def set_handler_message(name, dn, msg):
                 listener.unsetuid()
 
 
-def get_handler_message(name, binddn, bindpw):
-    # type: (str, str, str) -> dict
+def get_handler_message(name: str, binddn: str, bindpw: str) -> dict:
     msg = {}
     try:
         lo = udm_uldap.access(
@@ -164,59 +158,48 @@ def get_handler_message(name, binddn, bindpw):
         udm_modules.update()
         data_module = udm_modules.get('settings/data')
         udm_modules.init(lo, position, data_module)
-        data_object_dn = 'cn={},{}'.format(escape_dn_chars(name), position_dn)
+        data_object_dn = f'cn={escape_dn_chars(name)},{position_dn}'
         try:
             data_object = data_module.object(None, lo, position, dn=data_object_dn)
             data_object.open()
             if data_object.get('data', False):
                 msg = json.loads(bz2.decompress(base64.b64decode(data_object['data'])))
         except udm_errors.noObject:
-            msg = {"err": 'No object {} found'.format(data_object_dn)}
+            msg = {"err": f'No object {data_object_dn} found'}
     except Exception as err:
-        msg = {"err": 'Error get_handler_message for handler {}: {}'.format(name, err)}
+        msg = {"err": f'Error get_handler_message for handler {name}: {err}'}
     return msg
 
 
 class UniventionLDAPExtension(metaclass=ABCMeta):
 
     @abstractproperty
-    def udm_module_name(self):
-        # type: () -> str
+    def udm_module_name(self) -> str:
         pass
 
     @abstractproperty
-    def target_container_name(self):
-        # type: () -> str
+    def target_container_name(self) -> str:
         pass
 
     @abstractproperty
-    def active_flag_attribute(self):
-        # type: () -> str
+    def active_flag_attribute(self) -> str:
         pass
 
     @abstractproperty
-    def filesuffix(self):
-        # type: () -> str
+    def filesuffix(self) -> str:
         pass
 
-    def __init__(self, ucr):
-        # type: (ConfigRegistry) -> None
+    def __init__(self, ucr: ConfigRegistry) -> None:
         self.ucr = ucr
-        self._todo_list = []  # type: List[str]
+        self._todo_list: list[str] = []
         self.target_container_dn = "cn=%s,cn=univention,%s" % (escape_dn_chars(self.target_container_name), ucr["ldap/base"])
 
     @classmethod
-    def create_base_container(cls, ucr, udm_passthrough_options):
-        # type: (ConfigRegistry, List[str]) -> int
-        cmd = ["univention-directory-manager", "container/cn", "create"] + udm_passthrough_options + [
-            "--ignore_exists",
-            "--set", "name=%s" % cls.target_container_name,
-            "--position", "cn=univention,%s" % ucr['ldap/base'],
-        ]
+    def create_base_container(cls, ucr: ConfigRegistry, udm_passthrough_options: list[str]) -> int:
+        cmd = ['univention-directory-manager', 'container/cn', 'create', *udm_passthrough_options, '--ignore_exists', '--set', 'name=%s' % cls.target_container_name, '--position', 'cn=univention,%s' % ucr['ldap/base']]
         return subprocess.call(cmd)
 
-    def is_local_active(self):
-        # type: () -> Tuple[int, Optional[str]]
+    def is_local_active(self) -> tuple[int, str | None]:
         object_dn = None
 
         cmd = ["univention-ldapsearch", "-LLL", "-b", self.object_dn, "-s", "base", filter_format("(&(cn=%s)(%s=TRUE))", (self.objectname, self.active_flag_attribute))]
@@ -230,8 +213,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
             object_dn = m.group(1).decode('UTF-8')
         return (p.returncode, object_dn)
 
-    def is_applicable_for_current_ucs_version(self, ucr):
-        # type: (ConfigRegistry) -> bool
+    def is_applicable_for_current_ucs_version(self, ucr: ConfigRegistry) -> bool:
         current_ucs_version = "%s-%s" % (ucr.get('version/version'), ucr.get('version/patchlevel'))
         if self.options.ucsversionstart and UCS_Version(current_ucs_version) < UCS_Version(self.options.ucsversionstart):
             return False
@@ -239,8 +221,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
             return False
         return True  # probably yes
 
-    def wait_for_activation(self, timeout=180):
-        # type: (int) -> bool
+    def wait_for_activation(self, timeout: int = 180) -> bool:
         print("Waiting for activation of the extension object %s:" % (self.objectname,), end=' ')
         t0 = time.time()
         while not self.is_local_active()[1]:
@@ -254,17 +235,13 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
         print("OK")
         return True
 
-    def udm_find_object(self):
-        # type: () -> Tuple[int, str]
-        cmd = ["univention-directory-manager", self.udm_module_name, "list"] + self.udm_passthrough_options + [
-            "--filter", filter_format("name=%s", [self.objectname]),
-        ]
+    def udm_find_object(self) -> tuple[int, str]:
+        cmd = ['univention-directory-manager', self.udm_module_name, 'list', *self.udm_passthrough_options, '--filter', filter_format('name=%s', [self.objectname])]
         p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
         stdout, _ = p.communicate()
         return (p.returncode, stdout.decode('UTF-8', 'replace'))
 
-    def udm_find_object_dn(self):
-        # type: () -> Tuple[int, Optional[str], str]
+    def udm_find_object_dn(self) -> tuple[int, str | None, str]:
         object_dn = None
 
         rc, stdout = self.udm_find_object()
@@ -295,8 +272,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
         except udm_errors.base as exc:
             ud.debug(ud.LISTENER, ud.ERROR, 'Could not touch LDAP object %r: %s' % (self.object_dn, exc))
 
-    def register(self, filename, options, udm_passthrough_options, target_filename=None):
-        # type: (str, Values, List[str], Optional[str]) -> None
+    def register(self, filename: str, options: Values, udm_passthrough_options: list[str], target_filename: str | None = None) -> None:
         self.filename = filename
         self.options = options
         self.udm_passthrough_options = udm_passthrough_options
@@ -334,9 +310,9 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
         ]
 
         if self.udm_module_name == "settings/data":
-            common_udm_options.extend(["--set", "data_type={}".format(options.data_type)])
+            common_udm_options.extend(["--set", f"data_type={options.data_type}"])
             for meta in options.data_meta:
-                common_udm_options.extend(["--set", "meta={}".format(meta)])
+                common_udm_options.extend(["--set", f"meta={meta}"])
 
         if self.udm_module_name != "settings/ldapschema":
             if options.ucsversionstart:
@@ -387,10 +363,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
         rc, self.object_dn, stdout = self.udm_find_object_dn()
         if not self.object_dn:
 
-            cmd = ["univention-directory-manager", self.udm_module_name, "create"] + self.udm_passthrough_options + [
-                "--set", "name=%s" % self.objectname,
-                "--position", self.target_container_dn,
-            ] + common_udm_options + active_change_udm_options
+            cmd = ['univention-directory-manager', self.udm_module_name, 'create', *self.udm_passthrough_options, '--set', 'name=%s' % self.objectname, '--position', self.target_container_dn, *common_udm_options, *active_change_udm_options]
             p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
             out, _ = p.communicate()
             stdout = out.decode('UTF-8', 'replace')
@@ -403,10 +376,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
 
                 appidentifier = os.environ.get('UNIVENTION_APP_IDENTIFIER')
                 if appidentifier:
-                    cmd = ["univention-directory-manager", self.udm_module_name, "modify"] + self.udm_passthrough_options + [
-                        "--set", "appidentifier=%s" % (appidentifier,),
-                        "--dn", new_object_dn,
-                    ]
+                    cmd = ['univention-directory-manager', self.udm_module_name, 'modify', *self.udm_passthrough_options, '--set', 'appidentifier=%s' % (appidentifier,), '--dn', new_object_dn]
                     p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                     out, _ = p.communicate()
                     stdout = out.decode('UTF-8', 'replace')
@@ -466,9 +436,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
                 print("INFO: No change of core data of object %s." % (self.objectname,), file=sys.stderr)
                 active_change_udm_options = []
 
-            cmd = ["univention-directory-manager", self.udm_module_name, "modify"] + self.udm_passthrough_options + [
-                "--dn", self.object_dn,
-            ] + common_udm_options + active_change_udm_options
+            cmd = ['univention-directory-manager', self.udm_module_name, 'modify', *self.udm_passthrough_options, '--dn', self.object_dn, *common_udm_options, *active_change_udm_options]
             p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
             out, _ = p.communicate()
             stdout = out.decode('UTF-8', 'replace')
@@ -479,10 +447,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
 
             appidentifier = os.environ.get('UNIVENTION_APP_IDENTIFIER')
             if appidentifier:
-                cmd = ["univention-directory-manager", self.udm_module_name, "modify"] + self.udm_passthrough_options + [
-                    "--append", "appidentifier=%s" % (appidentifier,),
-                    "--dn", self.object_dn,
-                ]
+                cmd = ['univention-directory-manager', self.udm_module_name, 'modify', *self.udm_passthrough_options, '--append', 'appidentifier=%s' % (appidentifier,), '--dn', self.object_dn]
                 p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                 out, _ = p.communicate()
                 stdout = out.decode('UTF-8', 'replace')
@@ -494,8 +459,7 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
         if not self.object_dn:
             self.object_dn = new_object_dn
 
-    def unregister(self, objectname, options, udm_passthrough_options):
-        # type: (str, Values, List[str]) -> None
+    def unregister(self, objectname: str, options: Values, udm_passthrough_options: list[str]) -> None:
         self.objectname = objectname
         self.options = options
         self.udm_passthrough_options = udm_passthrough_options
@@ -525,16 +489,13 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
                 print("INFO: The object %s is still registered by the following apps: %s" % (objectname, apps), file=sys.stderr)
                 sys.exit(2)
 
-        cmd = ["univention-directory-manager", self.udm_module_name, "delete"] + self.udm_passthrough_options + [
-            "--dn", object_dn,
-        ]
+        cmd = ['univention-directory-manager', self.udm_module_name, 'delete', *self.udm_passthrough_options, '--dn', object_dn]
         p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         out, _ = p.communicate()
         stdout = out.decode('UTF-8', 'replace')
         print(stdout)
 
-    def mark_active(self, handler_name=None):
-        # type: (Optional[str]) -> None
+    def mark_active(self, handler_name: str | None = None) -> None:
         if self._todo_list:
             try:
                 lo, ldap_position = udm_uldap.getAdminConnection()
@@ -561,22 +522,20 @@ class UniventionLDAPExtension(metaclass=ABCMeta):
                 ud.debug(ud.LISTENER, ud.ERROR, 'Error accessing UDM: %s' % (e,))
                 if handler_name:
                     for object_dn in self._todo_list:
-                        set_handler_message(handler_name, object_dn, 'Error accessing UDM: {}'.format(e))
+                        set_handler_message(handler_name, object_dn, f'Error accessing UDM: {e}')
 
 
 class UniventionLDAPExtensionWithListenerHandler(UniventionLDAPExtension, metaclass=ABCMeta):
 
-    def __init__(self, ucr):
-        # type: (ConfigRegistry) -> None
-        super(UniventionLDAPExtensionWithListenerHandler, self).__init__(ucr)
+    def __init__(self, ucr: ConfigRegistry) -> None:
+        super().__init__(ucr)
         self._do_reload = False
         self.ucr_template_dir = '/etc/univention/templates'
         self.ucr_slapd_conf_subfile_dir = '%s/files/etc/ldap/slapd.conf.d' % self.ucr_template_dir
         self.ucr_info_basedir = '%s/info' % self.ucr_template_dir
 
     @abstractmethod
-    def handler(self, dn, new, old, name=""):
-        # type: (str, Dict[str, List[bytes]], Dict[str, List[bytes]], str) -> None
+    def handler(self, dn: str, new: dict[str, list[bytes]], old: dict[str, list[bytes]], name: str = "") -> None:
         pass
 
 
@@ -587,19 +546,16 @@ class UniventionLDAPSchema(UniventionLDAPExtensionWithListenerHandler):
     filesuffix = ".schema"
     basedir = '/var/lib/univention-ldap/local-schema'
 
-    def is_applicable_for_current_ucs_version(self, ucr):
-        # type: (ConfigRegistry) -> bool
+    def is_applicable_for_current_ucs_version(self, ucr: ConfigRegistry) -> bool:
         return True
 
-    def handler(self, dn, new, old, name=""):
-        # type: (str, Dict[str, List[bytes]], Dict[str, List[bytes]], str) -> None
+    def handler(self, dn: str, new: dict[str, list[bytes]], old: dict[str, list[bytes]], name: str = "") -> None:
         try:
             return self._handler(dn, new, old, name)
         except BaseDirRestriction as exc:
             ud.debug(ud.LISTENER, ud.ERROR, '%r basedir conflict: %s' % (dn, exc))
 
-    def _handler(self, dn, new, old, name=""):
-        # type: (str, Dict[str, List[bytes]], Dict[str, List[bytes]], str) -> None
+    def _handler(self, dn: str, new: dict[str, list[bytes]], old: dict[str, list[bytes]], name: str = "") -> None:
         """Handle LDAP schema extensions on Primary and Backup Directory Nodes"""
         if listener.configRegistry.get("server/role") not in ("domaincontroller_master", "domaincontroller_backup"):
             return
@@ -636,7 +592,7 @@ class UniventionLDAPSchema(UniventionLDAPExtensionWithListenerHandler):
                 new_object_data = bz2.decompress(new['univentionLDAPSchemaData'][0])
             except TypeError:
                 ud.debug(ud.LISTENER, ud.ERROR, '%s: Error uncompressing data of object %s.' % (name, dn))
-                set_handler_message(name, dn, 'Error uncompressing data of object {}.'.format(dn))
+                set_handler_message(name, dn, f'Error uncompressing data of object {dn}.')
                 return
 
             try:
@@ -678,7 +634,7 @@ class UniventionLDAPSchema(UniventionLDAPExtensionWithListenerHandler):
                         f.write(new_object_data)
                 except OSError:
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: Error writing file %s.' % (name, new_filename))
-                    set_handler_message(name, dn, 'Error writing file {}.'.format(new_filename))
+                    set_handler_message(name, dn, f'Error writing file {new_filename}.')
                     return
 
                 ucr = ConfigRegistry()
@@ -695,7 +651,7 @@ class UniventionLDAPSchema(UniventionLDAPExtensionWithListenerHandler):
                 stdout = out.decode('UTF-8', 'replace')
                 if p.returncode != 0:
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: validation failed (%s):\n%s.' % (name, p.returncode, stdout))
-                    set_handler_message(name, dn, 'slaptest validation failed {} {}'.format(stdout, p.returncode))
+                    set_handler_message(name, dn, f'slaptest validation failed {stdout} {p.returncode}')
                     # Revert changes
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: Removing new file %s.' % (name, new_filename))
                     os.unlink(new_filename)
@@ -733,7 +689,7 @@ class UniventionLDAPSchema(UniventionLDAPExtensionWithListenerHandler):
                         shutil.move(old_filename, backup_filename)
                     except OSError:
                         ud.debug(ud.LISTENER, ud.WARN, '%s: Error renaming old file %s, leaving it untouched.' % (name, old_filename))
-                        set_handler_message(name, dn, 'Error renaming old file {}, leaving it untouched.'.format(old_filename))
+                        set_handler_message(name, dn, f'Error renaming old file {old_filename}, leaving it untouched.')
                         os.close(backup_fd)
                         return
 
@@ -750,7 +706,7 @@ class UniventionLDAPSchema(UniventionLDAPExtensionWithListenerHandler):
                     stdout = out.decode('UTF-8', 'replace')
                     if p.returncode != 0:
                         ud.debug(ud.LISTENER, ud.ERROR, '%s: validation failed (%s):\n%s.' % (name, p.returncode, stdout))
-                        set_handler_message(name, dn, 'slaptest validation failed {} {}'.format(stdout, p.returncode))
+                        set_handler_message(name, dn, f'slaptest validation failed {stdout} {p.returncode}')
                         ud.debug(ud.LISTENER, ud.WARN, '%s: Restoring %s.' % (name, old_filename))
                         # Revert changes
                         try:
@@ -784,15 +740,13 @@ class UniventionLDAPACL(UniventionLDAPExtensionWithListenerHandler):
     filesuffix = ".acl"
     file_prefix = 'ldapacl_'
 
-    def handler(self, dn, new, old, name=""):
-        # type: (str, Dict[str, List[bytes]], Dict[str, List[bytes]], str) -> None
+    def handler(self, dn: str, new: dict[str, list[bytes]], old: dict[str, list[bytes]], name: str = "") -> None:
         try:
             return self._handler(dn, new, old, name)
         except BaseDirRestriction as exc:
             ud.debug(ud.LISTENER, ud.ERROR, '%r basedir conflict: %s' % (dn, exc))
 
-    def _handler(self, dn, new, old, name=""):
-        # type: (str, Dict[str, List[bytes]], Dict[str, List[bytes]], str) -> None
+    def _handler(self, dn: str, new: dict[str, list[bytes]], old: dict[str, list[bytes]], name: str = "") -> None:
         """Handle LDAP ACL extensions on Primary, Backup and Replica Directory Nodes"""
         if not listener.configRegistry.get('ldap/server/type'):
             return
@@ -857,7 +811,7 @@ class UniventionLDAPACL(UniventionLDAPExtensionWithListenerHandler):
                 new_object_data = bz2.decompress(new['univentionLDAPACLData'][0])
             except TypeError:
                 ud.debug(ud.LISTENER, ud.ERROR, '%s: Error uncompressing data of object %s.' % (name, dn))
-                set_handler_message(name, dn, 'Error uncompressing data of object {}.'.format(dn))
+                set_handler_message(name, dn, f'Error uncompressing data of object {dn}.')
                 return
 
             new_basename = new['univentionLDAPACLFilename'][0].decode('UTF-8')
@@ -927,7 +881,7 @@ class UniventionLDAPACL(UniventionLDAPExtensionWithListenerHandler):
                         f.write(new_object_data)
                 except OSError:
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: Error writing file %s.' % (name, new_filename))
-                    set_handler_message(name, dn, 'Error writing file {}.'.format(new_filename))
+                    set_handler_message(name, dn, f'Error writing file {new_filename}.')
                     return
 
                 # plus backlink file
@@ -938,7 +892,7 @@ class UniventionLDAPACL(UniventionLDAPExtensionWithListenerHandler):
                         f.write("%s\n" % dn)
                 except OSError:
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: Error writing backlink file %s.' % (name, new_backlink_filename))
-                    set_handler_message(name, dn, 'Error writing backlink file {}.'.format(new_backlink_filename))
+                    set_handler_message(name, dn, f'Error writing backlink file {new_backlink_filename}.')
                     return
 
                 # and UCR registration
@@ -949,7 +903,7 @@ class UniventionLDAPACL(UniventionLDAPExtensionWithListenerHandler):
                         f.write("Type: multifile\nMultifile: etc/ldap/slapd.conf\n\nType: subfile\nMultifile: etc/ldap/slapd.conf\nSubfile: etc/ldap/slapd.conf.d/%s\n" % new_basename)
                 except OSError:
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: Error writing UCR info file %s.' % (name, new_ucrinfo_filename))
-                    set_handler_message(name, dn, 'Error writing UCR info file {}.'.format(new_ucrinfo_filename))
+                    set_handler_message(name, dn, f'Error writing UCR info file {new_ucrinfo_filename}.')
                     return
 
                 # Commit to slapd.conf
@@ -966,7 +920,7 @@ class UniventionLDAPACL(UniventionLDAPExtensionWithListenerHandler):
                 stdout = out.decode('UTF-8', 'replace')
                 if p.returncode != 0:
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: slapd.conf validation failed:\n%s.' % (name, stdout))
-                    set_handler_message(name, dn, 'slaptest validation failed {} {}'.format(stdout, p.returncode))
+                    set_handler_message(name, dn, f'slaptest validation failed {stdout} {p.returncode}')
                     # Revert changes
                     ud.debug(ud.LISTENER, ud.ERROR, '%s: Removing new file %s.' % (name, new_filename))
                     os.unlink(new_filename)
@@ -1057,16 +1011,14 @@ class UniventionDataExtension(UniventionLDAPExtension):
     active_flag_attribute = ''
     filesuffix = ''
 
-    def is_local_active(self):
-        # type: () -> Tuple[int, Optional[str]]
+    def is_local_active(self) -> tuple[int, str | None]:
         """
         There is nothing to activate for a data extension,
         just pretend that everything is fine.
         """
         return (0, "foo")
 
-    def wait_for_activation(self, timeout=180):
-        # type: (int) -> bool
+    def wait_for_activation(self, timeout: int = 180) -> bool:
         return True
 
 
@@ -1075,14 +1027,12 @@ class UniventionUDMExtension(UniventionLDAPExtension, metaclass=ABCMeta):
     target_subdir = ''
 
     @property
-    def target_filepath(self):
-        # type: () -> str
+    def target_filepath(self) -> str:
         """return the most likely path where the listener will write the file to"""
         return os.path.abspath(os.path.join(os.path.dirname(udm.__file__), self.target_subdir, self.target_filename.replace('/', '')))
 
-    def wait_for_activation(self, timeout=180):
-        # type: (int) -> bool
-        if not super(UniventionUDMExtension, self).wait_for_activation(timeout):
+    def wait_for_activation(self, timeout: int = 180) -> bool:
+        if not super().wait_for_activation(timeout):
             return False
 
         target_filepath = self.target_filepath
@@ -1110,14 +1060,12 @@ class UniventionUDMModule(UniventionUDMExtension):
     target_subdir = 'handlers'
 
     @property
-    def target_filepath(self):
-        # type: () -> str
+    def target_filepath(self) -> str:
         """return the most likely path where the listener will write the file to"""
         module_dir, module_name = self.target_udm_module.split('/', 1)
         return os.path.abspath(os.path.join(os.path.dirname(udm.__file__), self.target_subdir, module_dir, '%s.py' % (module_name.replace('/', ''),)))
 
-    def register(self, filename, options, udm_passthrough_options, target_filename=None):
-        # type: (str, Values, List[str], Optional[str]) -> None
+    def register(self, filename: str, options: Values, udm_passthrough_options: list[str], target_filename: str | None = None) -> None:
         # Determine UDM module name
         saved_value = sys.dont_write_bytecode
         sys.dont_write_bytecode = True
@@ -1139,9 +1087,8 @@ class UniventionUDMModule(UniventionUDMExtension):
 
         UniventionUDMExtension.register(self, filename, options, udm_passthrough_options, target_filename=module_name + ".py")
 
-    def wait_for_activation(self, timeout=180):
-        # type: (int) -> bool
-        if not super(UniventionUDMModule, self).wait_for_activation(timeout):
+    def wait_for_activation(self, timeout: int = 180) -> bool:
+        if not super().wait_for_activation(timeout):
             return False
 
         timeout = 60
@@ -1175,23 +1122,20 @@ class UniventionUDMHook(UniventionUDMExtension):
     target_subdir = 'hooks.d'
 
 
-def option_validate_existing_filename(option, opt, value):
-    # type: (Option, str, str) -> str
+def option_validate_existing_filename(option: Option, opt: str, value: str) -> str:
     if not os.path.exists(value):
         raise OptionValueError("%s: file does not exist: %s" % (opt, value))
     return value
 
 
-def option_validate_ucs_version(option, opt, value):
-    # type: (Option, str, str) -> str
+def option_validate_ucs_version(option: Option, opt: str, value: str) -> str:
     regex = re.compile("[-.0-9]+")
     if not regex.match(value):
         raise OptionValueError("%s: may only contain digit, dot and dash characters: %s" % (opt, value))
     return value
 
 
-def option_validate_gnu_message_catalogfile(option, opt, value):
-    # type: (Option, str, str) -> str
+def option_validate_gnu_message_catalogfile(option: Option, opt: str, value: str) -> str:
     if not os.path.exists(value):
         raise OptionValueError("%s: file does not exist: %s" % (opt, value))
     filename_parts = os.path.splitext(value)
@@ -1205,15 +1149,14 @@ def option_validate_gnu_message_catalogfile(option, opt, value):
 
 
 class UCSOption(Option):
-    TYPES = Option.TYPES + ("existing_filename", "ucs_version")
+    TYPES = (*Option.TYPES, 'existing_filename', 'ucs_version')
     TYPE_CHECKER = copy(Option.TYPE_CHECKER)
     TYPE_CHECKER["existing_filename"] = option_validate_existing_filename
     TYPE_CHECKER["ucs_version"] = option_validate_ucs_version
     TYPE_CHECKER["gnu_message_catalogfile"] = option_validate_gnu_message_catalogfile
 
 
-def option_callback_udm_passthrough_options(option, opt_str, value, parser, *args):
-    # type: (Option, str, str, OptionParser, *List[str]) -> None
+def option_callback_udm_passthrough_options(option: Option, opt_str: str, value: str, parser: OptionParser, *args: list[str]) -> None:
     assert parser.values is not None
     assert option.dest is not None
     if value.startswith('--'):
@@ -1224,8 +1167,7 @@ def option_callback_udm_passthrough_options(option, opt_str, value, parser, *arg
     setattr(parser.values, option.dest, value)
 
 
-def check_data_module_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def check_data_module_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     if value.startswith('--'):
         raise OptionValueError("%s requires an argument" % (opt_str,))
@@ -1233,24 +1175,21 @@ def check_data_module_options(option, opt_str, value, parser):
         raise OptionValueError("%s can only be used after --data" % (opt_str,))
 
 
-def option_callback_set_data_module_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def option_callback_set_data_module_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     assert option.dest is not None
     check_data_module_options(option, opt_str, value, parser)
     setattr(parser.values, option.dest, value)
 
 
-def option_callback_append_data_module_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def option_callback_append_data_module_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     assert option.dest is not None
     check_data_module_options(option, opt_str, value, parser)
     parser.values.ensure_value(option.dest, []).append(value)
 
 
-def check_udm_module_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def check_udm_module_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     if value.startswith('--'):
         raise OptionValueError("%s requires an argument" % (opt_str,))
@@ -1258,24 +1197,21 @@ def check_udm_module_options(option, opt_str, value, parser):
         raise OptionValueError("%s can only be used after --udm_module" % (opt_str,))
 
 
-def option_callback_set_udm_module_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def option_callback_set_udm_module_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     assert option.dest is not None
     check_udm_module_options(option, opt_str, value, parser)
     setattr(parser.values, option.dest, value)
 
 
-def option_callback_append_udm_module_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def option_callback_append_udm_module_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     assert option.dest is not None
     check_udm_module_options(option, opt_str, value, parser)
     parser.values.ensure_value(option.dest, []).append(value)
 
 
-def check_udm_syntax_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def check_udm_syntax_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     if value.startswith('--'):
         raise OptionValueError("%s requires an argument" % (opt_str,))
@@ -1283,16 +1219,14 @@ def check_udm_syntax_options(option, opt_str, value, parser):
         raise OptionValueError("%s can only be used after --udm_syntax" % (opt_str,))
 
 
-def option_callback_append_udm_syntax_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def option_callback_append_udm_syntax_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     assert option.dest is not None
     check_udm_syntax_options(option, opt_str, value, parser)
     parser.values.ensure_value(option.dest, []).append(value)
 
 
-def check_udm_hook_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def check_udm_hook_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     if value.startswith('--'):
         raise OptionValueError("%s requires an argument" % (opt_str,))
@@ -1300,16 +1234,14 @@ def check_udm_hook_options(option, opt_str, value, parser):
         raise OptionValueError("%s can only be used after --udm_hook" % (opt_str,))
 
 
-def option_callback_append_udm_hook_options(option, opt_str, value, parser):
-    # type: (Option, str, str, OptionParser) -> None
+def option_callback_append_udm_hook_options(option: Option, opt_str: str, value: str, parser: OptionParser) -> None:
     assert parser.values is not None
     assert option.dest is not None
     check_udm_hook_options(option, opt_str, value, parser)
     parser.values.ensure_value(option.dest, []).append(value)
 
 
-def ucs_registerLDAPExtension():
-    # type: () -> None
+def ucs_registerLDAPExtension() -> None:
     functionname = inspect.stack()[0][3]
     parser = OptionParser(prog=functionname, option_class=UCSOption)
 
@@ -1422,7 +1354,7 @@ def ucs_registerLDAPExtension():
 
     # parser.add_option("-v", "--verbose", action="count")
 
-    udm_passthrough_options = []  # type: List[str]
+    udm_passthrough_options: list[str] = []
     auth_options = OptionGroup(parser, "Authentication Options", "These options are usually passed e.g. from a calling joinscript")
     auth_options.add_option(
         "--binddn", dest="binddn", type="string",
@@ -1456,7 +1388,7 @@ def ucs_registerLDAPExtension():
     ucr = ConfigRegistry()
     ucr.load()
 
-    objects = []  # type: List[UniventionLDAPExtension]
+    objects: list[UniventionLDAPExtension] = []
     if opts.schemafile:
         if UniventionLDAPSchema.create_base_container(ucr, udm_passthrough_options) != 0:
             sys.exit(1)
@@ -1524,8 +1456,7 @@ def ucs_registerLDAPExtension():
         p.wait()
 
 
-def ucs_unregisterLDAPExtension():
-    # type: () -> None
+def ucs_unregisterLDAPExtension() -> None:
     functionname = inspect.stack()[0][3]
     parser = OptionParser(prog=functionname, option_class=UCSOption)
 
@@ -1561,7 +1492,7 @@ def ucs_unregisterLDAPExtension():
 
     # parser.add_option("-v", "--verbose", action="count")
 
-    udm_passthrough_options = []  # type: List[str]
+    udm_passthrough_options: list[str] = []
     auth_options = OptionGroup(parser, "Authentication Options", "These options are usually passed e.g. from a calling joinscript")
     auth_options.add_option(
         "--binddn", dest="binddn", type="string",
