@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -35,10 +34,10 @@
 
 import re
 import sys
-from typing import List, Tuple, Union  # noqa: F401
+from typing import Self
 
 
-class UCS_Version(object):
+class UCS_Version:
     """Version object consisting of major-, minor-number and patch-level"""
 
     FORMAT = '%(major)d.%(minor)d'
@@ -47,8 +46,7 @@ class UCS_Version(object):
     _regexp = re.compile(r'(?P<major>[0-9]+)\.(?P<minor>[0-9]+)-(?P<patch>[0-9]+)')
     _format = re.compile(r'%([%afimp])')
 
-    def __init__(self, version):
-        # type: (Union[Tuple[int, int, int], List[int], str, UCS_Version]) -> None
+    def __init__(self, version: tuple[int, int, int] | list[int] | str | Self) -> None:
         """
         :param version: must a :py:class:`str` matching the pattern `X.Y-Z` or a triple with major, minor and patchlevel.
         :type version: list(int) or tuple(int) or str or UCS_Version
@@ -62,7 +60,7 @@ class UCS_Version(object):
         >>> UCS_Version(v) == v
         True
         """
-        if isinstance(version, (tuple, list)):
+        if isinstance(version, tuple | list):
             self.mmp = map(int, version)  # type: ignore
         elif isinstance(version, str):
             self.set(version)
@@ -72,24 +70,20 @@ class UCS_Version(object):
             raise TypeError("not a tuple, list or string")
 
     @property
-    def mm(self):
-        # type: () -> Tuple[int, int]
+    def mm(self) -> tuple[int, int]:
         """2-tuple (major, minor) version"""
         return (self.major, self.minor)
 
     @property
-    def mmp(self):
-        # type: () -> Tuple[int, int, int]
+    def mmp(self) -> tuple[int, int, int]:
         """3-tuple (major, minor, patch-level) version"""
         return (self.major, self.minor, self.patchlevel)
 
     @mmp.setter
-    def mmp(self, mmp):
-        # type: (Union[List[int], Tuple[int, int, int]]) -> None
+    def mmp(self, mmp: list[int] | tuple[int, int, int]) -> None:
         (self.major, self.minor, self.patchlevel) = mmp
 
-    def __lt__(self, other):
-        # type: (UCS_Version) -> bool
+    def __lt__(self, other: Self) -> bool:
         """
         Compare to UCS versions.
 
@@ -104,8 +98,7 @@ class UCS_Version(object):
         """
         return self.mmp < other.mmp if isinstance(other, UCS_Version) else NotImplemented
 
-    def __le__(self, other):
-        # type: (UCS_Version) -> bool
+    def __le__(self, other: Self) -> bool:
         """
         >>> UCS_Version((1, 2, 3)) <= UCS_Version((1, 2, 3))
         True
@@ -114,8 +107,7 @@ class UCS_Version(object):
         """
         return self.mmp <= other.mmp if isinstance(other, UCS_Version) else NotImplemented
 
-    def __eq__(self, other):
-        # type: (object) -> bool
+    def __eq__(self, other: object) -> bool:
         """
         >>> UCS_Version((1, 0, 0)) == UCS_Version((1, 0, 0))
         True
@@ -124,8 +116,7 @@ class UCS_Version(object):
         """
         return isinstance(other, UCS_Version) and self.mmp == other.mmp
 
-    def __ne__(self, other):
-        # type: (object) -> bool
+    def __ne__(self, other: object) -> bool:
         """
         >>> UCS_Version((1, 0, 0)) != UCS_Version((1, 0, 0))
         False
@@ -134,8 +125,7 @@ class UCS_Version(object):
         """
         return not isinstance(other, UCS_Version) or self.mmp != other.mmp
 
-    def __ge__(self, other):
-        # type: (UCS_Version) -> bool
+    def __ge__(self, other: Self) -> bool:
         """
         >>> UCS_Version((1, 2, 3)) >= UCS_Version((1, 2, 3))
         True
@@ -144,8 +134,7 @@ class UCS_Version(object):
         """
         return self.mmp >= other.mmp if isinstance(other, UCS_Version) else NotImplemented
 
-    def __gt__(self, other):
-        # type: (UCS_Version) -> bool
+    def __gt__(self, other: Self) -> bool:
         """
         >>> UCS_Version((1, 2, 3)) > UCS_Version((1, 2, 3))
         False
@@ -154,8 +143,7 @@ class UCS_Version(object):
         """
         return self.mmp > other.mmp if isinstance(other, UCS_Version) else NotImplemented
 
-    def set(self, version):
-        # type: (str) -> None
+    def set(self, version: str) -> None:
         """
         Parse string and set version.
 
@@ -167,13 +155,11 @@ class UCS_Version(object):
             raise ValueError('string %s does not match UCS version pattern' % version)
         self.mmp = map(int, match.groups())  # type: ignore
 
-    def __getitem__(self, k):
-        # type: (str) -> int
+    def __getitem__(self, k: str) -> int:
         """Dual natured dictionary: retrieve value from attribute."""
         return self.__dict__[k]
 
-    def __str__(self):
-        # type: () -> str
+    def __str__(self) -> str:
         """
         Return full version string.
 
@@ -182,12 +168,10 @@ class UCS_Version(object):
         """
         return self.FULLFORMAT % self
 
-    def __hash__(self):
-        # type: () -> int
+    def __hash__(self) -> int:
         return hash(self.mmp)
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         """
         Return canonical string representation.
 
@@ -196,8 +180,7 @@ class UCS_Version(object):
         """
         return 'UCS_Version((%d,%d,%r))' % self.mmp
 
-    def __format__(self, fmt):
-        # type: (str) -> str
+    def __format__(self, fmt: str) -> str:
         """
         Support Format String Syntax:
 
