@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Like what you see? Join us!
 # https://www.univention.com/about-us/careers/vacancies/
@@ -46,20 +45,20 @@ connections to remote |UMC| servers
 """
 
 import base64
-import http  # noqa: F401
+import http
 import json
 import locale
 import ssl
 from http.client import HTTPException, HTTPSConnection
 # the SameSite cookie attribute is only available from Python 3.8
 from http.cookies import Morsel, SimpleCookie
-from typing import Any, Dict, List, Optional, Tuple, Type, TypeVar, Union, overload  # noqa: F401
+from typing import Any, Self, TypeVar, overload
 
 from univention.config_registry import ConfigRegistry
 
 
 Morsel._reserved['samesite'] = 'SameSite'
-_T = TypeVar("_T")  # noqa: PYI018
+_T = TypeVar("_T")
 
 ucr = ConfigRegistry()
 ucr.load()
@@ -87,9 +86,8 @@ class ConnectionError(Exception):
     :param reason: The optional underlying exception.
     """
 
-    def __init__(self, msg, reason=None):
-        # type: (str, Exception) -> None
-        super(ConnectionError, self).__init__(msg, reason)
+    def __init__(self, msg: str, reason: Exception | None = None) -> None:
+        super().__init__(msg, reason)
         self.reason = reason
 
 
@@ -103,12 +101,11 @@ class HTTPError(Exception, metaclass=_HTTPType):
     :param str hostname: The host name of the failed server.
     """
 
-    codes = {}  # type: Dict[int, Type[HTTPError]]
+    codes: dict[int, type['HTTPError']] = {}
     """Specialized sub-classes for individual |HTTP| error codes."""
 
     @property
-    def status(self):
-        # type: () -> int
+    def status(self) -> int:
         """
         Return the |HTTP| status code.
 
@@ -118,8 +115,7 @@ class HTTPError(Exception, metaclass=_HTTPType):
         return self.response.status
 
     @property
-    def message(self):
-        # type: () -> str
+    def message(self) -> str:
         """
         Return the |HTTP| status message.
 
@@ -129,8 +125,7 @@ class HTTPError(Exception, metaclass=_HTTPType):
         return self.response.message
 
     @property
-    def result(self):
-        # type: () -> str
+    def result(self) -> str:
         """
         Return the |HTTP| result.
 
@@ -141,19 +136,17 @@ class HTTPError(Exception, metaclass=_HTTPType):
 
     def __new__(cls, request, response, hostname):
         err = cls.codes.get(response.status, cls)
-        return super(HTTPError, cls).__new__(err, request, response, hostname)  # type: ignore
+        return super().__new__(err, request, response, hostname)  # type: ignore
 
     def __init__(self, request, response, hostname):
         self.request = request
         self.hostname = hostname
         self.response = response
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         return '<HTTPError %s>' % (self,)
 
-    def __str__(self):
-        # type: () -> str
+    def __str__(self) -> str:
         traceback = ''
         data = self.response.data
         if self.status >= 500 and isinstance(self.response.data, dict) and isinstance(self.response.data.get('traceback'), str) and 'Traceback (most recent call last)' in self.response.data['traceback']:
@@ -252,7 +245,7 @@ class ServiceUnavailable(HTTPError):
     code = 503
 
 
-class Request(object):
+class Request:
     """
     The |HTTP| request.
 
@@ -262,15 +255,13 @@ class Request(object):
     :param dict headers: a mapping of HTTP headers
     """
 
-    def __init__(self, method, path, data=None, headers=None):
-        # type: (str, str, Optional[bytes], Optional[Dict[str, str]]) -> None
+    def __init__(self, method: str, path: str, data: bytes | None = None, headers: dict[str, str] | None = None) -> None:
         self.method = method
         self.path = path
         self.data = data
         self.headers = headers or {}
 
-    def get_body(self):
-        # type: () -> Optional[bytes]
+    def get_body(self) -> bytes | None:
         """
         Return the request data.
 
@@ -282,7 +273,7 @@ class Request(object):
         return self.data
 
 
-class Response(object):
+class Response:
     """
     The |HTTP| response.
 
@@ -294,8 +285,7 @@ class Response(object):
     """
 
     @property
-    def result(self):
-        # type: () -> Any
+    def result(self) -> Any:
         """
         Return `result` from |JSON| data.
 
@@ -305,8 +295,7 @@ class Response(object):
             return self.data.get('result')
 
     @property
-    def message(self):
-        # type: () -> Any
+    def message(self) -> Any:
         """
         Return `message` from |JSON| data.
 
@@ -315,8 +304,7 @@ class Response(object):
         if isinstance(self.data, dict):
             return self.data.get('message')
 
-    def __init__(self, status, reason, body, headers, _response):
-        # type: (int, str, bytes, List[Tuple[str, str]], http.client.HTTPResponse) -> None
+    def __init__(self, status: int, reason: str, body: bytes, headers: list[tuple[str, str]], _response: http.client.HTTPResponse) -> None:
         self.status = status
         self.reason = reason
         self.body = body
@@ -325,17 +313,14 @@ class Response(object):
         self.data = self.decode_body()
 
     @overload
-    def get_header(self, name):
-        # type: (str) -> Optional[str]
+    def get_header(self, name: str) -> str | None:
         pass
 
     @overload
-    def get_header(self, name, default=None):
-        # type: (str, _T) -> _T
+    def get_header(self, name: str, default: _T = None) -> _T:
         pass
 
-    def get_header(self, name, default=None):
-        # type: (str, _T) -> Union[None, str, _T]
+    def get_header(self, name: str, default: _T = None) -> None | str | _T:
         """
         Return original |HTTP| response header.
 
@@ -346,8 +331,7 @@ class Response(object):
         """
         return self._response.getheader(name, default)
 
-    def decode_body(self):
-        # type: () -> Union[bytes, dict]
+    def decode_body(self) -> bytes | dict:
         """
         Decode |HTTP| response and return |JSON| data as dictionary.
 
@@ -363,8 +347,7 @@ class Response(object):
         return data
 
     @classmethod
-    def _from_httplib_response(cls, response):
-        # type: (http.client.HTTPResponse) -> Response
+    def _from_httplib_response(cls, response: http.client.HTTPResponse) -> Self:
         """
         Create class instance from |HTTP| response.
 
@@ -374,7 +357,7 @@ class Response(object):
         return cls(response.status, response.reason, data, response.getheaders(), response)
 
 
-class Client(object):
+class Client:
     """
     A client capable to speak with a |UMC| server.
 
@@ -388,8 +371,7 @@ class Client(object):
 
     ConnectionType = HTTPSConnection
 
-    def __init__(self, hostname=None, username=None, password=None, language=None, timeout=None, automatic_reauthentication=False):
-        # type: (Optional[str], Optional[str], Optional[str], Optional[str], Optional[float], bool) -> None
+    def __init__(self, hostname: str | None = None, username: str | None = None, password: str | None = None, language: str | None = None, timeout: float | None = None, automatic_reauthentication: bool = False) -> None:
         self.hostname = hostname or '%(hostname)s.%(domainname)s' % ucr
         self._language = language or locale.getlocale()[0] or ''
         self._headers = {
@@ -403,14 +385,13 @@ class Client(object):
         self._timeout = timeout
         self._raise_errors = True
         self._automatic_reauthentication = automatic_reauthentication
-        self.cookies = {}  # type: Dict[str, str]
+        self.cookies: dict[str, str] = {}
         self.username = username or ''
         self.password = password or ''
         if username:
             self.authenticate(self.username, self.password)
 
-    def authenticate(self, username, password):
-        # type: (str, str) -> Response
+    def authenticate(self, username: str, password: str) -> Response:
         """
         Authenticate against the host and preserves the
         cookie. Has to be done only once (but keep in mind that the
@@ -423,13 +404,11 @@ class Client(object):
         self.password = password
         return self.umc_auth(username, password)
 
-    def reauthenticate(self):
-        # type: () -> Response
+    def reauthenticate(self) -> Response:
         """Re-authenticate using the stored username and password."""
         return self.authenticate(self.username, self.password)
 
-    def set_basic_http_authentication(self, username, password):
-        # type: (str, str) -> None
+    def set_basic_http_authentication(self, username: str, password: str) -> None:
         """
         Setup authentication using |HTTP| Basic authentication.
 
@@ -438,8 +417,7 @@ class Client(object):
         """
         self._headers['Authorization'] = 'Basic %s' % (base64.b64encode(b'%s:%s' % (username.encode('UTF-8'), password.encode('UTF-8'))).decode('ASCII'),)
 
-    def authenticate_saml(self, username, password):
-        # type: (str, str) -> None
+    def authenticate_saml(self, username: str, password: str) -> None:
         """
         Setup authentication using |SAML|.
 
@@ -451,8 +429,7 @@ class Client(object):
         """
         raise ConnectionError('SAML authentication currently not supported.')
 
-    def authenticate_with_machine_account(self):
-        # type: () -> None
+    def authenticate_with_machine_account(self) -> None:
         """
         Setup authentication using the machine account.
 
@@ -466,8 +443,7 @@ class Client(object):
             raise ConnectionError('Could not read /etc/machine.secret', reason=exc)
         self.authenticate(username, password)
 
-    def umc_command(self, path, options=None, flavor=None, headers=None):
-        # type: (str, Optional[dict], Optional[str], Optional[dict]) -> Response
+    def umc_command(self, path: str, options: dict | None = None, flavor: str | None = None, headers: dict | None = None) -> Response:
         """
         Perform generic |UMC| command.
 
@@ -481,8 +457,7 @@ class Client(object):
         data = self.__build_data(options, flavor)
         return self.request('POST', 'command/%s' % (path,), data, headers)
 
-    def umc_set(self, options, headers=None):
-        # type: (Optional[dict], Optional[dict]) -> Response
+    def umc_set(self, options: dict | None, headers: dict | None = None) -> Response:
         """
         Perform |UMC| `set` command.
 
@@ -495,8 +470,7 @@ class Client(object):
         return self.request('POST', 'set', data, headers)
         # TODO: return self.request('POST', 'set/%s' % options.keys()[0], data, headers)
 
-    def umc_set_password(self, options, headers=None):
-        # type: (Optional[dict], Optional[dict]) -> Response
+    def umc_set_password(self, options: dict | None, headers: dict | None = None) -> Response:
         """
         Perform |UMC| `set/password` command. Target UMC version need to be >= UCS 5.0-4.
 
@@ -508,8 +482,7 @@ class Client(object):
         data = self.__build_data(options)
         return self.request('POST', 'set/password', data, headers)
 
-    def umc_get(self, path, options=None, headers=None):
-        # type: (str, Optional[dict], Optional[dict]) -> Response
+    def umc_get(self, path: str, options: dict | None = None, headers: dict | None = None) -> Response:
         """
         Perform |UMC| `get` command.
 
@@ -521,8 +494,7 @@ class Client(object):
         """
         return self.request('POST', 'get/%s' % path, self.__build_data(options), headers)
 
-    def umc_upload(self):
-        # type: () -> None
+    def umc_upload(self) -> None:
         """
         Perform |UMC| upload action.
 
@@ -531,8 +503,7 @@ class Client(object):
         """
         raise NotImplementedError('File uploads currently need to be done manually.')
 
-    def umc_auth(self, username, password, **data):
-        # type: (str, str, **str) -> Response
+    def umc_auth(self, username: str, password: str, **data: str) -> Response:
         """
         Perform |UMC| authentication command.
 
@@ -545,8 +516,7 @@ class Client(object):
         data = self.__build_data(dict({'username': username, 'password': password}, **data))
         return self.request('POST', 'auth', data)
 
-    def umc_logout(self):
-        # type: () -> Response
+    def umc_logout(self) -> Response:
         """
         Perform |UMC| logout action.
 
@@ -558,8 +528,7 @@ class Client(object):
         except (SeeOther, Found, MovedPermanently) as exc:
             return exc.response
 
-    def request(self, method, path, data=None, headers=None):
-        # type: (str, str, Any, Optional[dict]) -> Response
+    def request(self, method: str, path: str, data: Any = None, headers: dict | None = None) -> Response:
         """
         Send request to |UMC| server handling re-authentication.
 
@@ -580,8 +549,7 @@ class Client(object):
             self.reauthenticate()
             return self.send(request)
 
-    def send(self, request):
-        # type: (Request) -> Response
+    def send(self, request: Request) -> Response:
         """
         Low-level function to send request to |UMC| server.
 
@@ -605,20 +573,18 @@ class Client(object):
             raise HTTPError(request, umc_response, self.hostname)
         return umc_response
 
-    def _handle_cookies(self, response):
-        # type: (http.client.HTTPResponse) -> None
+    def _handle_cookies(self, response: http.client.HTTPResponse) -> None:
         """
         Parse cookies from |HTTP| response and store for next request.
 
         :param http.client.HTTPResponse: The |HTTP| response.
         """
         # FIXME: this cookie handling doesn't respect path, domain and expiry
-        cookies = SimpleCookie()  # type: SimpleCookie[Any]
+        cookies: SimpleCookie[Any] = SimpleCookie()
         cookies.load(response.getheader('set-cookie', ''))
         self.cookies.update({cookie.key: cookie.value for cookie in cookies.values()})
 
-    def __request(self, request):
-        # type: (Request) -> http.client.HTTPResponse
+    def __request(self, request: Request) -> http.client.HTTPResponse:
         """
         Perform a request to the |UMC| server and return its response.
 
@@ -641,8 +607,7 @@ class Client(object):
                 return self.__request(request)
         return response
 
-    def _get_connection(self):
-        # type: () -> HTTPSConnection
+    def _get_connection(self) -> HTTPSConnection:
         """
         Creates a new connection to the host.
 
@@ -653,8 +618,7 @@ class Client(object):
         #   so create a new connection on every request
         return self.ConnectionType(self.hostname, timeout=self._timeout)
 
-    def __build_data(self, data, flavor=None):
-        # type: (Optional[Dict[str, Any]], Optional[str]) -> Dict[str, Any]
+    def __build_data(self, data: dict[str, Any] | None, flavor: str | None = None) -> dict[str, Any]:
         """
         Create a dictionary as expected by the |UMC| Server.
 
